@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Canonical, Ltd.
+ * Copyright (C) 2010-2012 Canonical, Ltd.
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License
@@ -17,6 +17,7 @@
  *
  * Authored by
  *             Mikkel Kamstrup Erlandsen <mikkel.kamstrup@canonical.com>
+ *             Michal Hruby <michal.hruby@canonical.com>
  */
 
 #if HAVE_CONFIG_H
@@ -66,6 +67,11 @@ typedef struct
   /* Method calls queued up while waiting for a proxy  */
   GSList *method_dispatch_queue;
 
+  /* Are we connected to Zeitgeist? */
+  gboolean is_connected;
+
+  /* Cached variant with daemon version info */
+  GVariant *engine_version;
 } ZeitgeistLogPrivate;
 
 /* Property ids */
@@ -93,9 +99,9 @@ static void    _zeitgeist_log_on_zg_appeared    (GDBusConnection *connection,
                                                  const gchar     *name_owner,
                                                  gpointer         user_data);
 
-static void    _zeitgeist_log_on_zg_vanished    (GDBusConnection *connection,
-                                                 const gchar *name,
-                                                 gpointer user_data);
+static void    _zeitgeist_log_on_name_owner_changed (GObject *proxy,
+                                                     GParamSpec *pspec,
+                                                     gpointer user_data);
 
 static void    _zeitgeist_log_on_zg_proxy_acquired (GObject *source_object,
                                                     GAsyncResult *res,
@@ -117,22 +123,25 @@ static void    dispatch_async_callback (GObject               *source_object,
                                         GAsyncResult          *res,
                                         gpointer               user_data);
 
+static ZeitgeistLog *zeitgeist_log_singleton = NULL;
+
 static void
 zeitgeist_log_init (ZeitgeistLog *self)
 {
   ZeitgeistLogPrivate *priv;
 
   priv = ZEITGEIST_LOG_GET_PRIVATE (self);
-  
+  priv->log = NULL;
+
   /* Reset hash set of monitors */
   priv->monitors = g_hash_table_new (g_direct_hash, g_direct_equal);
-  
+
   /* Set up the connection to the ZG daemon */
   priv->watcher_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                        "org.gnome.zeitgeist.Engine",
                                        G_BUS_NAME_WATCHER_FLAGS_AUTO_START,
                                        _zeitgeist_log_on_zg_appeared,
-                                       _zeitgeist_log_on_zg_vanished,
+                                       NULL,
                                        self,
                                        NULL);
 }
@@ -171,6 +180,11 @@ zeitgeist_log_finalize (GObject *object)
                             log);
       g_hash_table_unref (priv->monitors);
     }
+
+  if (priv->engine_version)
+    {
+      g_variant_unref (priv->engine_version);
+    }
   
   G_OBJECT_CLASS (zeitgeist_log_parent_class)->finalize (object); 
 }
@@ -182,11 +196,12 @@ zeitgeist_log_get_property (GObject    *object,
                             GParamSpec *pspec)
 {
   ZeitgeistLogPrivate *priv = ZEITGEIST_LOG_GET_PRIVATE (object);
+  gchar *name_owner;
 
   switch (prop_id)
     {
       case PROP_CONNECTED:
-        g_value_set_boolean (value, priv->log != NULL);
+        g_value_set_boolean (value, priv->is_connected);
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -305,6 +320,10 @@ dispatch_async_callback (GObject      *source_object,
  * until the connection is ready - at which point you will get a notify signal
  * on the #ZeitgeistLog:connected property.
  *
+ * It isn't recommended to call this function more than once. If you need to
+ * access #ZeitgeistLog from different parts of your codebase, consider using
+ * zeitgeist_log_get_default() instead.
+ *
  * Returns: A reference to a newly allocated log.
  */
 ZeitgeistLog*
@@ -318,6 +337,26 @@ zeitgeist_log_new (void)
   return log;
 }
 
+/**
+ * zeitgeist_log_get_default:
+ * Get a unique instance of #ZeitgeistLog, that you can share in your
+ * application without caring about memory management.
+ *
+ * See zeitgeist_log_new() for more information.
+ *
+ * Returns: (transfer none): A unique #ZeitgeistLog. Do not ref or unref it.
+ *
+ * Since: 0.3.14
+ */
+ZeitgeistLog*
+zeitgeist_log_get_default (void)
+{
+  if (G_UNLIKELY (!zeitgeist_log_singleton))
+    zeitgeist_log_singleton = zeitgeist_log_new ();
+
+  return zeitgeist_log_singleton;
+}
+
 /**
  * zeitgeist_log_insert_events:
  * @self: The log logging the events
@@ -329,7 +368,7 @@ zeitgeist_log_new (void)
  * @user_data: Any user data to pass back to @callback
  * @VarArgs: A list of #ZeitgeistEvent<!-- -->s terminated by a %NULL
  *
- * Asynchrnously send a set of events to the Zeitgeist daemon, requesting they
+ * Asynchronously send a set of events to the Zeitgeist daemon, requesting they
  * be inserted into the log.
  */
 void
@@ -352,9 +391,9 @@ zeitgeist_log_insert_events (ZeitgeistLog        *self,
  * @self: The log logging the events
  * @VarArgs: A list of #ZeitgeistEvent<!-- -->s terminated by a %NULL
  *
- * Asynchrnously send a set of events to the Zeitgeist daemon, requesting they
+ * Asynchronously send a set of events to the Zeitgeist daemon, requesting they
  * be inserted into the log. This method is &quot;fire and forget&quot; and the
- * caller will never know whether the events was succesfully inserted or not.
+ * caller will never know whether the events was successfully inserted or not.
  *
  * This method is exactly equivalent to calling zeitgeist_log_insert_events()
  * with %NULL set as @cancellable, @callback, and @user_data.
@@ -387,7 +426,7 @@ zeitgeist_log_insert_events_no_reply (ZeitgeistLog *self,
  * from C code it's generally more handy to use zeitgeist_log_insert_events()
  * or zeitgeist_log_insert_events_from_ptrarray().
  * 
- * Asynchrnously send a set of events to the Zeitgeist daemon, requesting they
+ * Asynchronously send a set of events to the Zeitgeist daemon, requesting they
  * be inserted into the log.
  */
 void
@@ -727,7 +766,52 @@ zeitgeist_log_get_events_finish (ZeitgeistLog        *self,
   g_variant_unref (_val);
 
   return _zeitgeist_simple_result_set_new (events, events->len);
-}                          
+}
+
+/**
+ * zeitgeist_log_get_version:
+ * @self: A #ZeitgeistLog instance
+ * @major: (out): Location for the major version
+ * @minor: (out): Location for the minor version
+ * @micro: (out): Location for the micro version
+ * 
+ * Gets version of currently running Zeitgeist daemon.
+ *
+ * This method will return the version of Zeitgeist daemon this instance is
+ * connected to. If you call this method right after zeitgeist_log_new(),
+ * only zeros will be returned, a valid version number will only be returned
+ * once this instance successfully connected to the Zeitgeist daemon - ie
+ * the value of the "connected" property must be TRUE (you can connect
+ * to the "notify::connected" signal otherwise).
+ */
+void
+zeitgeist_log_get_version (ZeitgeistLog *self,
+                           gint *major,
+                           gint *minor,
+                           gint *micro)
+{
+  ZeitgeistLogPrivate *priv;
+  gint maj, min, mic;
+
+  g_return_if_fail (ZEITGEIST_IS_LOG (self));
+  priv = ZEITGEIST_LOG_GET_PRIVATE (self);
+
+  if (priv->engine_version && g_variant_is_of_type (priv->engine_version,
+                                                    G_VARIANT_TYPE ("(iii)")))
+  {
+    g_variant_get (priv->engine_version, "(iii)", &maj, &min, &mic);
+  }
+  else
+  {
+    maj = 0;
+    min = 0;
+    mic = 0;
+  }
+
+  if (major) *major = maj;
+  if (minor) *minor = min;
+  if (micro) *micro = mic;
+}
 
 void
 zeitgeist_log_delete_events (ZeitgeistLog        *self,
@@ -984,15 +1068,18 @@ _zeitgeist_log_on_zg_appeared (GDBusConnection *connection,
       priv->connection = g_object_ref (connection);
     }
 
-  g_dbus_proxy_new (connection,
-                    G_DBUS_PROXY_FLAGS_NONE,
-                    NULL,
-                    name_owner,
-                    "/org/gnome/zeitgeist/log/activity",
-                    "org.gnome.zeitgeist.Log",
-                    NULL,
-                    _zeitgeist_log_on_zg_proxy_acquired,
-                    g_object_ref (self));
+  if (priv->log == NULL)
+    {
+      g_dbus_proxy_new (connection,
+                        G_DBUS_PROXY_FLAGS_NONE,
+                        NULL,
+                        name,
+                        "/org/gnome/zeitgeist/log/activity",
+                        "org.gnome.zeitgeist.Log",
+                        NULL,
+                        _zeitgeist_log_on_zg_proxy_acquired,
+                        g_object_ref (self));
+    }
 }
 
 /* Async callback for when a proxy is ready
@@ -1006,6 +1093,7 @@ _zeitgeist_log_on_zg_proxy_acquired (GObject *source_object,
   ZeitgeistLogPrivate *priv;
   GHashTableIter       iter;
   gpointer             monitor, dummy;
+  GVariant            *version_property;
   GError              *error;
 
   self = ZEITGEIST_LOG (user_data);
@@ -1030,6 +1118,16 @@ _zeitgeist_log_on_zg_proxy_acquired (GObject *source_object,
       goto cleanup;
     }
 
+  version_property = g_dbus_proxy_get_cached_property (priv->log, "version");
+  if (version_property)
+    {
+      if (priv->engine_version) g_variant_unref (priv->engine_version);
+      priv->engine_version = version_property; // steals the ref
+    }
+
+  g_signal_connect (priv->log, "notify::g-name-owner",
+      G_CALLBACK (_zeitgeist_log_on_name_owner_changed), self);
+
   /* Reinstate all active monitors */
   g_hash_table_iter_init (&iter, priv->monitors);
   while (g_hash_table_iter_next (&iter, &monitor, &dummy))
@@ -1037,6 +1135,7 @@ _zeitgeist_log_on_zg_proxy_acquired (GObject *source_object,
       _zeitgeist_log_install_monitor (self, ZEITGEIST_MONITOR (monitor));
     }
 
+  priv->is_connected = TRUE;
   g_object_notify (G_OBJECT (self), "connected");
 
   /* Dispatch all queued method calls we got while we didn't have a proxy.
@@ -1052,20 +1151,50 @@ _zeitgeist_log_on_zg_proxy_acquired (GObject *source_object,
 
 /* Called when the Zeitgeist daemon leaves the bus */
 static void
-_zeitgeist_log_on_zg_vanished (GDBusConnection *connection,
-                               const gchar *name,
-                               gpointer user_data)
+_zeitgeist_log_on_name_owner_changed (GObject *proxy,
+                                      GParamSpec *pspec,
+                                      gpointer user_data)
 {
   ZeitgeistLog        *self;
   ZeitgeistLogPrivate *priv;
+  gchar               *name_owner;
+  gboolean             connected;
+  GHashTableIter       iter;
+  GVariant            *version_property;
+  gpointer             monitor, dummy;
 
   self = ZEITGEIST_LOG (user_data);
   priv = ZEITGEIST_LOG_GET_PRIVATE (self);
 
-  g_object_unref (priv->log);
-  priv->log = NULL;
+  name_owner = g_dbus_proxy_get_name_owner (G_DBUS_PROXY (proxy));
+  connected = name_owner != NULL;
 
-  g_object_notify (G_OBJECT (self), "connected");
+  if (connected)
+    {
+      /* Reinstate all active monitors */
+      g_hash_table_iter_init (&iter, priv->monitors);
+      while (g_hash_table_iter_next (&iter, &monitor, &dummy))
+        {
+          _zeitgeist_log_install_monitor (self, ZEITGEIST_MONITOR (monitor));
+        }
+
+      /* Update our cached version property */
+      version_property = g_dbus_proxy_get_cached_property (priv->log,
+                                                           "version");
+      if (version_property)
+        {
+          if (priv->engine_version) g_variant_unref (priv->engine_version);
+          priv->engine_version = version_property; // steals the ref
+        }
+    }
+
+  if (priv->is_connected ^ connected) /* XOR here! */
+    {
+      priv->is_connected = connected;
+      g_object_notify (G_OBJECT (self), "connected");
+    }
+
+  if (name_owner) g_free (name_owner);
 }
 
 gboolean
@@ -1074,7 +1203,7 @@ zeitgeist_log_is_connected (ZeitgeistLog *self)
   ZeitgeistLogPrivate *priv;
 
   priv = ZEITGEIST_LOG_GET_PRIVATE (self);
-  return priv->log != NULL;
+  return priv->is_connected;
 }
 
 /*
