@@ -310,6 +310,7 @@ _ensure_mimes_loaded (void)
   zeitgeist_register_mimetype ("text/x-chdr", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("application/x-gzip", ZEITGEIST_NFO_ARCHIVE);
   zeitgeist_register_mimetype ("application/x-dia-diagram", ZEITGEIST_NFO_SOURCE_CODE);
+  zeitgeist_register_mimetype ("image/vnd.djvu", ZEITGEIST_NFO_PAGINATED_TEXT_DOCUMENT);
   zeitgeist_register_mimetype ("application/x-csh", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("text/css", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("text/x-csharp", ZEITGEIST_NFO_SOURCE_CODE);
@@ -325,6 +326,7 @@ _ensure_mimes_loaded (void)
   zeitgeist_register_mimetype ("application/vnd.ms-excel", ZEITGEIST_NFO_SPREADSHEET);
   zeitgeist_register_mimetype ("text/x-dsrc", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("text/x-haskell", ZEITGEIST_NFO_SOURCE_CODE);
+  zeitgeist_register_mimetype ("inode/directory", ZEITGEIST_NFO_FOLDER);
   zeitgeist_register_mimetype ("application/x-object", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("text/x-c", ZEITGEIST_NFO_SOURCE_CODE);
   zeitgeist_register_mimetype ("application/x-ruby", ZEITGEIST_NFO_SOURCE_CODE);
@@ -358,11 +360,14 @@ _ensure_schemes_loaded (void)
   if (schemes_loaded) return;
 
   zeitgeist_register_uri_scheme ("file://", ZEITGEIST_NFO_FILE_DATA_OBJECT);
-  zeitgeist_register_uri_scheme ("http://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
-  zeitgeist_register_uri_scheme ("https://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
+  zeitgeist_register_uri_scheme ("http://", ZEITGEIST_NFO_WEB_DATA_OBJECT);
+  zeitgeist_register_uri_scheme ("https://", ZEITGEIST_NFO_WEB_DATA_OBJECT);
   zeitgeist_register_uri_scheme ("ssh://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
   zeitgeist_register_uri_scheme ("sftp://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
   zeitgeist_register_uri_scheme ("ftp://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
+  zeitgeist_register_uri_scheme ("dav://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
+  zeitgeist_register_uri_scheme ("davs://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
+  zeitgeist_register_uri_scheme ("smb://", ZEITGEIST_NFO_REMOTE_DATA_OBJECT);
 
   schemes_loaded = TRUE;
 }
