@@ -43,6 +43,8 @@ typedef enum
   ZEITGEIST_SUBJECT_MIMETYPE,
   ZEITGEIST_SUBJECT_ORIGIN,
   ZEITGEIST_SUBJECT_TEXT,
+  ZEITGEIST_SUBJECT_STORAGE,
+  ZEITGEIST_SUBJECT_CURRENT_URI,
 } ZeitgeistSubjectDataOffset;
 
 typedef struct
@@ -54,6 +56,7 @@ typedef struct
   gchar *origin;
   gchar *text;
   gchar *storage;
+  gchar *current_uri;
 } ZeitgeistSubjectPrivate;
 
 const gchar*
@@ -245,6 +248,68 @@ zeitgeist_subject_set_storage (ZeitgeistSubject *subject,
   priv->storage = copy;
 }
 
+/**
+ * zeitgeist_subject_get_current_uri:
+ * @subject: The subject to get the current_uri from
+ *
+ * Get the current_uri of a subject.
+ *
+ * This is the updated URI taking into account possible relocations of
+ * the resource. It is illegal to insert an event with subjects having
+ * a `current_uri' different than their `uri'.
+ *
+ * A special case is events with #ZEITGEIST_ZG_MOVE_EVENT interpretation.
+ * In this case, `current_uri' is the destination to which `uri' is being
+ * moved, and they are expected to be different.
+ *
+ * Returns: The current_uri of @subject.
+ *
+ * Since: 0.3.14
+ */
+const gchar*
+zeitgeist_subject_get_current_uri (ZeitgeistSubject *subject)
+{
+  g_return_val_if_fail (ZEITGEIST_IS_SUBJECT (subject), NULL);
+
+  ZeitgeistSubjectPrivate *priv = ZEITGEIST_SUBJECT_GET_PRIVATE (subject);
+  return priv->current_uri;
+}
+
+/**
+ * zeitgeist_subject_set_current_uri:
+ * @subject: The subject to set the current_uri for
+ * @current_uri: The current_uri to set
+ *
+ * Set the current_uri of a subject.
+ *
+ * This is the updated URI taking into account possible relocations of
+ * the resource. It is illegal to insert an event with subjects having
+ * a `current_uri' different than their `uri'.
+ *
+ * A special case is events with #ZEITGEIST_ZG_MOVE_EVENT interpretation.
+ * In this case, `current_uri' is the destination to which `uri' is being
+ * moved, and they are expected to be different.
+ *
+ * Since: 0.3.14
+ */
+void
+zeitgeist_subject_set_current_uri (ZeitgeistSubject *subject,
+                                   const gchar *current_uri)
+{
+  g_return_if_fail (ZEITGEIST_IS_SUBJECT (subject));
+
+  ZeitgeistSubjectPrivate *priv = ZEITGEIST_SUBJECT_GET_PRIVATE (subject);
+
+  gchar* copy = g_strdup (current_uri);
+
+  if (priv->current_uri)
+    {
+      g_free (priv->current_uri);
+    }
+
+  priv->current_uri = copy;
+}
+
 static void
 zeitgeist_subject_init (ZeitgeistSubject *object)
 {
@@ -258,6 +323,7 @@ zeitgeist_subject_init (ZeitgeistSubject *object)
   priv->origin = NULL;
   priv->text = NULL;
   priv->storage = NULL;
+  priv->current_uri = NULL;
 }
 
 static void
@@ -275,6 +341,7 @@ zeitgeist_subject_finalize (GObject *object)
   zeitgeist_subject_set_origin (subject, NULL);
   zeitgeist_subject_set_text (subject, NULL);
   zeitgeist_subject_set_storage (subject, NULL);
+  zeitgeist_subject_set_current_uri (subject, NULL);
   
   G_OBJECT_CLASS (zeitgeist_subject_parent_class)->finalize (object);
 }
@@ -306,6 +373,7 @@ zeitgeist_subject_new (void)
   return g_object_new (ZEITGEIST_TYPE_SUBJECT, NULL);
 }
 
+// FIXME: add `current_uri' parameter
 /** 
  * zeitgeist_subject_new_full:
  * @uri: The URI or URL of the subject
@@ -319,7 +387,6 @@ zeitgeist_subject_new (void)
  * @text: A small textual representation of the subject suitable for display
  * @storage: String identifier for the storage medium the subject is on.
  *           Se zeitgeist_subject_set_storage() for details
- * 
  * Create a new subject structure with predefined data
  *
  * Returns: A newly create #ZeitgeistSubject instance. The returned subject will
@@ -347,6 +414,6 @@ zeitgeist_subject_new_full (const gchar *uri,
   zeitgeist_subject_set_origin (self, origin);
   zeitgeist_subject_set_text (self, text);
   zeitgeist_subject_set_storage (self, storage);
-    
+
   return self;
 }
